@@ -7,10 +7,12 @@ struct timeout {
 
     timeout() : finite(false) {}
 
-    timeout(unsigned int const duration) : duration(duration) {}
+    timeout(unsigned int const duration) :
+        finite(true), duration(duration) {}
+        // 'duration' is the timeout time in milliseconds
 
     bool finite;
-    unsigned int duration;
+    unsigned int duration;  // in milliseconds
 };
 
 
