@@ -1,21 +1,29 @@
 #ifndef HTTP_H_INCLUDED
 #define HTTP_H_INCLUDED
 
+#include "bool.h"
 #include "conn.h"
 
 /*********************************************************************
 ** Request
 *********************************************************************/
 
-abyss_bool RequestValidURI(TSession *r);
-abyss_bool RequestValidURIPath(TSession *r);
-abyss_bool RequestUnescapeURI(TSession *r);
+bool RequestValidURI(TSession * const r);
+bool RequestValidURIPath(TSession * const r);
+bool RequestUnescapeURI(TSession *r);
 
-abyss_bool RequestRead(TSession *r);
-void RequestInit(TSession *r,TConn *c);
-void RequestFree(TSession *r);
+void
+RequestRead(TSession * const sessionP,
+            uint32_t   const timeout);
 
-abyss_bool RequestAuth(TSession *r,char *credential,char *user,char *pass);
+void RequestInit(TSession * const r,TConn * const c);
+void RequestFree(TSession * const r);
+
+bool
+RequestAuth(TSession *   const sessionP,
+            const char * const credential,
+            const char * const user,
+            const char * const pass);
 
 /*********************************************************************
 ** HTTP
@@ -29,18 +37,18 @@ HTTPRead(TSession *   const sessionP,
          const char * const buffer,
          uint32_t     const len);
 
-abyss_bool
+bool
 HTTPWriteBodyChunk(TSession *   const sessionP,
                    const char * const buffer,
                    uint32_t     const len);
 
-abyss_bool
+bool
 HTTPWriteEndChunk(TSession * const sessionP);
 
-abyss_bool
+bool
 HTTPKeepalive(TSession * const sessionP);
 
-abyss_bool
+bool
 HTTPWriteContinue(TSession * const sessionP);
 
 #endif
