@@ -2,6 +2,7 @@
 #define SESSION_H_INCLUDED
 
 #include "xmlrpc-c/abyss.h"
+#include "bool.h"
 #include "date.h"
 #include "data.h"
 
@@ -11,31 +12,39 @@ typedef struct {
 } httpVersion;
 
 struct _TSession {
-    abyss_bool validRequest;
+    bool validRequest;
         /* Client has sent, and server has recognized, a valid HTTP request.
            This is false when the session is new.  If and when the server
            reads the request from the client and finds it to be valid HTTP,
            it becomes true.
         */
-    TRequestInfo request_info;
+    TRequestInfo requestInfo;
+        /* Some of the strings this references are in individually malloc'ed
+           memory, but some are pointers into arbitrary other data structures
+           that happen to live as long as the session.  Some day, we will
+           fix that.
+
+           'requestInfo' is valid only if 'validRequest' is true.
+        */
     uint32_t nbfileds;
     TList cookies;
     TList ranges;
 
     uint16_t status;
-        /* Response status from handler.  Zero means handler has not
-           set it.
+        /* Response status from handler.  Zero means session is not ready
+           for a response yet.  This can mean that we ran a handler and it
+           did not call ResponseStatus() to declare this fact.
         */
     TString header;
 
-    abyss_bool serverDeniesKeepalive;
+    bool serverDeniesKeepalive;
         /* Server doesn't want keepalive for this session, regardless of
            what happens in the session.  E.g. because the connection has
            already been kept alive long enough.
         */
-    abyss_bool responseStarted;
+    bool responseStarted;
         /* Handler has at least started the response (i.e. called
-           ResponseWrite())
+           ResponseWriteStart())
         */
 
     struct _TConn * conn;
@@ -43,14 +52,19 @@ struct _TSession {
     httpVersion version;
 
     TTable request_headers;
+        /* All the headers in the HTTP request.  The key is the header
+           name in lower case.  The value is the verbatim value from
+           the header.
+        */
+
     TTable response_headers;
 
-    TDate date;
+    time_t date;
 
-    abyss_bool chunkedwrite;
-    abyss_bool chunkedwritemode;
+    bool chunkedwrite;
+    bool chunkedwritemode;
 
-    abyss_bool continueRequired;
+    bool continueRequired;
         /* This client must receive 100 (continue) status before it will
            send more of the body of the request.
         */
