@@ -1,6 +1,9 @@
+#include "bool.h"
+
 #include "xmlrpc-c/sleep_int.h"
 
 #ifdef WIN32
+#include <windows.h>
 #include <process.h>
 #else
 #include <unistd.h>
@@ -11,7 +14,7 @@ void
 xmlrpc_millisecond_sleep(unsigned int const milliseconds) {
 
 #ifdef WIN32
-    Sleep(milliseconds);
+    SleepEx(milliseconds, true);
 #else
     usleep(milliseconds * 1000);
 #endif
